import logging
from .actions import oauth_post_request, oauth_request, send_message
from .datauri import DataURI
import urllib


def process_events(event_list, resource, token, config):
    for event in event_list:
        try:
            rel = event['link']['rel']
            func = EVENT_MAP[rel]
            func(event, resource, token, config)
        except KeyError:
            logging.warn("Not sure how to process - %s" % rel)
            logging.debug(event)


def process_message_invitation_event(message, resource, token, config):
    print('accepting messaging invite from %s' % message['_embedded']['messagingInvitation']['_embedded']['from']['name'])
    if str(message.get('status', '')) == 'Success':
        logging.debug('Received successful conversation acceptance from sender.')
        return
    try:
        accept_uri = message['_embedded']['messagingInvitation']['_links']['accept']['href']
        oauth_post_request(resource + accept_uri, token, config['redirect_uri'], {})
        logging.info('Accepted invitation')

    except KeyError:
        logging.error('Failed message')


def process_communication_event(message, resource, token, config):
    print('communication event')
    message_type = str(message['type'])
    if message_type == 'updated':
        # get existing conversations
        conversations_uri = message['_embedded']['communication']['_links']['conversations']['href']
        conversations = oauth_request(resource + conversations_uri, token, config['redirect_uri'])
    pass


def process_conversation_event(message, resource, token, config):
    message_type = str(message['type'])
    conversation_url = message['link']['href']

    logging.debug('%s conversation at %s' % (message_type, conversation_url))
    conversation_uri = message['link']['href']

    conversation = oauth_request(resource + conversation_uri, token, config['redirect_uri'])

    messaging_uri = conversation['_links']['messaging']['href']

    if message_type == 'added':
        thread_id = message['_embedded']['conversation']['threadId']
        send_message(resource + messaging_uri + '/messages', 'hi', token, config['redirect_uri'])


def process_missed_items_event(message, resource, token, config):
    print('missed items event')
    pass


def process_message_event(message, resource, token, config):
    logging.debug('Processing message event')
    try:
        if str(message['_embedded']['message']['direction']) == 'Incoming':
            message_uri = message['_embedded']['message']['_links']['plainMessage']['href']
            logging.debug("Received raw message - %s" % message_uri)

            inbound_message = urllib.unquote_plus(DataURI(message_uri).data)
            logging.info("Received message - %s" % inbound_message)

            thread_uri = message['_embedded']['message']['_links']['messaging']['href']

            if MESSAGE_CALLBACK is not None:
                MESSAGE_CALLBACK(inbound_message, thread_uri, resource)
            # send_message(resource + thread_uri + '/messages', 'I found 4 matching incidents https://it12321.servicenow.com/search?query={0}'.format(inbound_message), token, config['redirect_uri'])
    except KeyError:
        logging.debug('not an inbound message')
    pass

EVENT_MAP = {
    'messagingInvitation': process_message_invitation_event,
    'communication': process_communication_event,
    'conversation': process_conversation_event,
    'missedItems': process_missed_items_event,
    'message': process_message_event,
}

MESSAGE_CALLBACK = None


from flask import Flask, redirect, request
from future.moves.urllib.parse import urlparse

from .config import load_config
from .actions import (do_autodiscover, do_user_discovery)
from .auth import grant_flow_token
import yaml

app = Flask(__name__)

config = load_config()


@app.route('/')
def main():
    return "Hello."


@app.route('/autodiscover')
def autodiscover():
    return redirect('https://webdir.online.lync.com/autodiscover/autodiscoverservice.svc/root')


@app.route('/token', methods=['POST'])
def token_stage():
    # find out where this account is
    xframe, user_discovery_uri, resource = do_autodiscover(config['domain'])

    # Get the inbound token as posted from a form
    token = request.form['access_token']
    state = request.form['session_state']

    # discover which web server we have been assigned to
    user_discovery_data = do_user_discovery(resource, token, config)

    # Link to the current session server resource (we need a token for this.)
    instance_url = user_discovery_data['_links']['applications']['href']

    instance_parts = urlparse(instance_url)
    instance_resource = '{0}://{1}'.format(instance_parts.scheme, instance_parts.netloc)

    url = grant_flow_token(config['client_id'],
                                     config['redirect_uri'] + '/directsession',
                                     instance_resource, state, token)

    return redirect(url)


@app.route('/directsession', methods=['GET', 'POST'])
def direct_sesssion_stage():
    # Get the inbound token as posted from a form
    token = request.form['access_token']
    resource = request.form['state']

    # app = register_application(resource, token, config)
    # r = do_application_discovery(resource, token, config)

    app = {
        'token': str(token),
        'resource': str(resource)
    }

    with open('instance.yml', 'w') as instance_f:
        instance_f.write(yaml.dump(app))

    return 'done. now run app.py'

app.run()


import uuid
import requests
from future.moves.urllib.parse import urlparse
import json


USER_AGENT = 'SkypeWeb/0.4.275 master UCWA/1.0.0-e4a1abbd7ca7d-84c26b3703da8'


def do_autodiscover(domain):
    r = requests.get('https://webdir.online.lync.com/autodiscover/autodiscoverservice.svc/root?originalDomain=%s' % domain)
    discovery = r.json()
    path = discovery['_links']['user']['href']
    domain = urlparse(path)
    host = '{0}://{1}'.format(domain.scheme, domain.netloc)
    return (discovery['_links']['xframe']['href'], path, host)


def do_user_discovery(resource, token, config):
    return oauth_request(
        resource + '/Autodiscover/AutodiscoverService.svc/root/oauth/user', token,
        config['redirect_uri'])


def do_application_discovery(resource, token, config):
    return oauth_request(
        resource + '/ucwa/oauth/v1/applications', token,
        config['redirect_uri'])


def register_application(resource, token, config):
    msg = {
        "UserAgent": USER_AGENT,
        "EndpointId": str(uuid.uuid1()),
        "Culture": "en-US"
       }
    return oauth_post_request(
        resource + '/ucwa/oauth/v1/applications', token,
        config['redirect_uri'], msg)


def set_available(resource, app_id, token, config):
    path = '/ucwa/oauth/v1/applications/%s/me/makeMeAvailable' % app_id
    msg = {
        "SupportedModalities": ["Messaging"]
        }
    return oauth_post_request(
        resource + path, token,
        config['redirect_uri'], msg)


def oauth_post_request(uri, oauth_token, origin, msg):
    uid = uuid.uuid1()
    headers = {
        'Authorization': 'Bearer %s' % oauth_token,
        'Origin': origin,
        'Client-Request-Id': 'WebSDK/%s' % uid,
        'Accept': 'application/json',
        'Content-Type': 'application/json',
        'X-MS-Correlation-Id': uid,
        'X-Ms-Namespace': 'internal',
        'X-Ms-SDK-Instance': USER_AGENT,
        'Referer': origin + '/'
    }
    response = requests.post(uri, data=json.dumps(msg), headers=headers, verify=False)
    response.raise_for_status()
    if response.text != '':
        return response.json()
    else:
        return {}


def oauth_post_text_request(uri, oauth_token, origin, data):
    uid = uuid.uuid1()
    headers = {
        'Authorization': 'Bearer %s' % oauth_token,
        'Origin': origin,
        'Client-Request-Id': 'WebSDK/%s' % uid,
        'Accept': 'application/json',
        'Content-Type': 'text/plain',
        'X-MS-Correlation-Id': uid,
        'X-Ms-Namespace': 'internal',
        'X-Ms-SDK-Instance': USER_AGENT,
        'Referer': origin + '/'
    }
    response = requests.post(uri, data=data, headers=headers, verify=False)
    response.raise_for_status()
    if response.text != '':
        return response.json()
    else:
        return {}


def oauth_stream_request(uri, oauth_token, origin):
    uid = uuid.uuid1()
    headers = {
        'Authorization': 'Bearer %s' % oauth_token,
        'Origin': origin,
        'Client-Request-Id': 'WebSDK/%s' % uid,
        'Accept': 'application/json',
        'X-Requested-With': 'XMLHttpRequest',
        'X-MS-Correlation-Id': uid,
        'X-Ms-Namespace': 'internal',
        'X-Ms-SDK-Instance': USER_AGENT,
        'Referer': origin + '/'
    }
    response = requests.get(uri, headers=headers, stream=True, verify=False)
    response.raise_for_status()
    return response


def oauth_request(uri, oauth_token, origin):
    uid = uuid.uuid1()
    headers = {
        'Authorization': 'Bearer %s' % oauth_token,
        'Origin': origin,
        'Client-Request-Id': 'WebSDK/%s' % uid,
        'Accept': 'application/json',
        'X-Requested-With': 'XMLHttpRequest',
        'X-MS-Correlation-Id': uid,
        'X-Ms-Namespace': 'internal',
        'X-Ms-SDK-Instance': USER_AGENT,
        'Referer': origin + '/'
    }
    response = requests.get(uri, headers=headers, verify=False)
    response.raise_for_status()
    return response.json()


def send_message(uri, message, token, origin):
    return oauth_post_text_request(uri, token, origin, message)



from future.moves.urllib.parse import urlencode
import requests
from .actions import do_autodiscover
from .config import load_config


# Constant strings for OAuth2 flow
# The OAuth authority
authority = 'https://login.microsoftonline.com'

config = load_config()


def get_signin_url(redirect_uri, client_id, tenant, resource):
    xframe, user_discovery_uri, resource = do_autodiscover(config['domain'])

    # Build the query parameters for the signin url
    params = {
      'client_id': client_id,
      'redirect_uri': redirect_uri,
      'response_type': 'token',
      'response_mode': 'form_post',
      'resource': resource
    }

    # The authorize URL that initiates the OAuth2 client credential flow for admin consent
    authorize_url = '{0}{1}'.format(authority, '/%s/oauth2/authorize?{0}' % tenant)

    # Format the sign-in url for redirection
    signin_url = authorize_url.format(urlencode(params))

    return signin_url


def admin_consent(client_id, tenant, redirect_uri, resource, state):
    # Build the post form for the token request
    params = {
      'client_id': client_id,
      'redirect_uri': redirect_uri,
      'response_type': 'id_token',
      'response_mode': 'form_post',
      'resource': resource,
      'nonce': 'yousaywhat',
      'prompt': 'admin_consent'
    }

    # The token issuing endpoint
    authorize_url = '{0}{1}'.format(authority, '/%s/oauth2/authorize?{0}' % tenant)

    # Format the sign-in url for redirection
    url = authorize_url.format(urlencode(params))

    return url


def grant_flow_token(client_id, redirect_uri, resource, state, token):
    # Build the post form for the token request
    params = {
      'client_id': client_id,
      'redirect_uri': redirect_uri + '?resource=' + resource,
      'response_type': 'token',
      'state': resource,
      'resource': resource,
      'prompt': 'none',
      'response_mode': 'form_post',
    }

    # The token issuing endpoint
    authorize_url = '{0}{1}'.format(authority, '/common/oauth2/authorize?{0}')

    # Format the sign-in url for redirection
    url = authorize_url.format(urlencode(params))

    return url


def get_token_from_code(client_id, tenant, auth_code, redirect_uri, resource, client_secret):
    # Build the post form for the token request
    post_data = {
        'grant_type': 'authorization_code',
        'code': auth_code,
        'redirect_uri': redirect_uri,
        'resource': resource,
        'client_id': client_id,
        'client_secret': client_secret
      }

    # The token issuing endpoint
    token_url = '{0}{1}'.format(authority, '/{0}/oauth2/token'.format(tenant))

    # Perform the post to get access token
    response = requests.post(token_url, data=post_data)

    try:
        # try to parse the returned JSON for an access token
        access_token = response.json()['id_token']
        return access_token
    except:
        raise Exception('Error retrieving token: {0} - {1}'.format(
          response.status_code, response.text))
